__title__ = 'sciutil'
__description__ = 'A set of utility functions for the sci-packages.'
__url__ = 'https://github.com/arianemora/sciutil'
__version__ = '1.0.0'
__author__ = 'Ariane Mora'
__author_email__ = 'ariane.mora@uq.net.au'
__license__ = 'GPL3'